@@ -4,10 +4,9 @@ import re
 import shutil
 import sys
 from pathlib import Path
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING
 
 import pytest
-from sphinx.testing.path import path
 from sphobjinv import Inventory
 
 if TYPE_CHECKING:
@@ -32,27 +31,26 @@ def inv(pytestconfig: Config) -> Inventory:
 
 
 @pytest.fixture(autouse=True)
-def _remove_sphinx_projects(sphinx_test_tempdir: path) -> None:
+def _remove_sphinx_projects(sphinx_test_tempdir: Path) -> None:
     # Remove any directory which appears to be a Sphinx project from
     # the temporary directory area.
     # See https://github.com/sphinx-doc/sphinx/issues/4040
-    roots_path = Path(sphinx_test_tempdir)
-    for entry in roots_path.iterdir():
+    for entry in sphinx_test_tempdir.iterdir():
         try:
             if entry.is_dir() and Path(entry, "_build").exists():
                 shutil.rmtree(str(entry))
-        except PermissionError:
+        except PermissionError:  # noqa: PERF203
             pass
 
 
-@pytest.fixture()
-def rootdir() -> path:
-    return path(str(Path(__file__).parent) or ".").abspath() / "roots"
+@pytest.fixture
+def rootdir() -> Path:
+    return Path(str(Path(__file__).parent) or ".").absolute() / "roots"
 
 
-def pytest_ignore_collect(path: Any, config: Config) -> bool | None:  # noqa: ARG001
+def pytest_ignore_collect(collection_path: Path, config: Config) -> bool | None:  # noqa: ARG001
     version_re = re.compile(r"_py(\d)(\d)\.py$")
-    match = version_re.search(path.basename)
+    match = version_re.search(collection_path.name)
     if match:
         version = tuple(int(x) for x in match.groups())
         if sys.version_info < version:
