@@ -6,11 +6,20 @@ from dataclasses import dataclass
 from inspect import isclass
 from pathlib import Path
 from textwrap import dedent, indent
-from typing import TYPE_CHECKING, Any, Callable, Optional, TypeVar, Union, overload  # no type comments
+from typing import (  # no type comments
+    TYPE_CHECKING,
+    Any,
+    NewType,
+    Optional,
+    TypeVar,
+    Union,
+    overload,
+)
 
 import pytest
 
 if TYPE_CHECKING:
+    from collections.abc import AsyncGenerator, Callable
     from io import StringIO
     from mailbox import Mailbox
     from types import CodeType, ModuleType
@@ -18,11 +27,13 @@ if TYPE_CHECKING:
     from sphinx.testing.util import SphinxTestApp
 
 T = TypeVar("T")
+W = NewType("W", str)
 
 
-def expected(expected: str) -> Callable[[T], T]:
+def expected(expected: str, **options: dict[str, Any]) -> Callable[[T], T]:
     def dec(val: T) -> T:
         val.EXPECTED = expected
+        val.OPTIONS = options
         return val
 
     return dec
@@ -38,7 +49,7 @@ def warns(pattern: str) -> Callable[[T], T]:
 
 @expected("mod.get_local_function()")
 def get_local_function():  # noqa: ANN201
-    def wrapper(self) -> str:  # noqa: ANN001, ARG001
+    def wrapper(self) -> str:  # noqa: ANN001
         """
         Wrapper
         """
@@ -137,10 +148,10 @@ class Class:
     :param z: baz
     """
 
-    def __init__(self, x: bool, y: int, z: Optional[str] = None) -> None:  # noqa: UP007
+    def __init__(self, x: bool, y: int, z: Optional[str] = None) -> None:
         pass
 
-    def a_method(self, x: bool, y: int, z: Optional[str] = None) -> str:  # noqa: UP007
+    def a_method(self, x: bool, y: int, z: Optional[str] = None) -> str:
         """
         Method docstring.
 
@@ -163,7 +174,7 @@ class Class:
         :param x: foo
         """
 
-    def __magic_custom_method__(self, x: str) -> str:
+    def __magic_custom_method__(self, x: str) -> str:  # noqa: PLW3201
         """
         Magic dunder method docstring.
 
@@ -171,7 +182,7 @@ class Class:
         """
 
     @classmethod
-    def a_classmethod(cls, x: bool, y: int, z: Optional[str] = None) -> str:  # noqa: ANN102, UP007
+    def a_classmethod(cls, x: bool, y: int, z: Optional[str] = None) -> str:
         """
         Classmethod docstring.
 
@@ -181,7 +192,7 @@ class Class:
         """
 
     @staticmethod
-    def a_staticmethod(x: bool, y: int, z: Optional[str] = None) -> str:  # noqa: UP007
+    def a_staticmethod(x: bool, y: int, z: Optional[str] = None) -> str:
         """
         Staticmethod docstring.
 
@@ -259,7 +270,7 @@ mod.function(x, y, z_=None)
       bytes
 """,
 )
-def function(x: bool, y: int, z_: Optional[str] = None) -> str:  # noqa: ARG001, UP007
+def function(x: bool, y: int, z_: Optional[str] = None) -> str:
     """
     Function docstring.
 
@@ -287,7 +298,7 @@ mod.function_with_starred_documentation_param_names(*args, **kwargs)
       * ****kwargs** ("str") -- bar
 """,
 )
-def function_with_starred_documentation_param_names(*args: int, **kwargs: str):  # noqa: ANN201, ARG001
+def function_with_starred_documentation_param_names(*args: int, **kwargs: str):  # noqa: ANN201
     r"""
     Function docstring.
 
@@ -310,7 +321,7 @@ mod.function_with_escaped_default(x='\\\\x08')
       **x** ("str") -- foo
 """,
 )
-def function_with_escaped_default(x: str = "\b"):  # noqa: ANN201, ARG001
+def function_with_escaped_default(x: str = "\b"):  # noqa: ANN201
     """
     Function docstring.
 
@@ -326,10 +337,10 @@ mod.function_with_unresolvable_annotation(x)
    Function docstring.
 
    Parameters:
-      **x** (*a.b.c*) -- foo
+      **x** (a.b.c) -- foo
 """,
 )
-def function_with_unresolvable_annotation(x: a.b.c):  # noqa: ANN201, ARG001, F821
+def function_with_unresolvable_annotation(x: a.b.c):  # noqa: ANN201, F821
     """
     Function docstring.
 
@@ -353,8 +364,8 @@ mod.function_with_typehint_comment(x, y)
 """,
 )
 def function_with_typehint_comment(  # noqa: ANN201
-    x,  # type: int  # noqa: ANN001, ARG001
-    y,  # type: str  # noqa: ANN001, ARG001
+    x,  # type: int  # noqa: ANN001
+    y,  # type: str  # noqa: ANN001
 ):
     # type: (...) -> None
     """
@@ -403,7 +414,7 @@ class ClassWithTypehints:
         # type: (...) -> None
         pass
 
-    def foo(  # noqa: ANN201
+    def foo(  # noqa: ANN201, PLR6301
         self,
         x,  # type: str  # noqa: ANN001, ARG002
     ):
@@ -415,7 +426,7 @@ class ClassWithTypehints:
         """
         return 42
 
-    def method_without_typehint(self, x):  # noqa: ANN001, ANN201, ARG002
+    def method_without_typehint(self, x):  # noqa: ANN001, ANN201, ARG002, PLR6301
         """
         Method docstring.
         """
@@ -445,7 +456,7 @@ mod.function_with_typehint_comment_not_inline(x=None, *y, z, **kwargs)
       "None"
 """,
 )
-def function_with_typehint_comment_not_inline(x=None, *y, z, **kwargs):  # noqa: ANN001, ANN002, ANN003, ANN201, ARG001
+def function_with_typehint_comment_not_inline(x=None, *y, z, **kwargs):  # noqa: ANN001, ANN002, ANN003, ANN201
     # type: (Union[str, bytes, None], *str, bytes, **int) -> None
     """
     Function docstring.
@@ -498,7 +509,7 @@ class ClassWithTypehintsNotInline:
     def __init__(self, x=None) -> None:  # type: (Optional[Callable[[int, bytes], int]]) -> None  # noqa: ANN001
         pass
 
-    def foo(self, x=1):  # type: (Callable[[int, bytes], int]) -> int  # noqa: ANN001, ANN201
+    def foo(self, x=1):  # type: (Callable[[int, bytes], int]) -> int  # noqa: ANN001, ANN201, PLR6301
         """
         Method docstring.
 
@@ -508,7 +519,7 @@ class ClassWithTypehintsNotInline:
 
     @classmethod
     def mk(  # noqa: ANN206
-        cls,  # noqa: ANN102
+        cls,
         x=None,  # noqa: ANN001
     ):  # type: (Optional[Callable[[int, bytes], int]]) -> ClassWithTypehintsNotInline
         """
@@ -580,7 +591,7 @@ mod.mocked_import(x)
       **x** ("Mailbox") -- function
 """,
 )
-def mocked_import(x: Mailbox):  # noqa: ANN201, ARG001
+def mocked_import(x: Mailbox):  # noqa: ANN201
     """
     A docstring.
 
@@ -613,13 +624,11 @@ def func_with_examples() -> int:
 
 
 @overload
-def func_with_overload(a: int, b: int) -> None:
-    ...
+def func_with_overload(a: int, b: int) -> None: ...
 
 
 @overload
-def func_with_overload(a: str, b: str) -> None:
-    ...
+def func_with_overload(a: str, b: str) -> None: ...
 
 
 @expected(
@@ -638,7 +647,7 @@ mod.func_with_overload(a, b)
       "None"
 """,
 )
-def func_with_overload(a: Union[int, str], b: Union[int, str]) -> None:  # noqa: ARG001, UP007
+def func_with_overload(a: Union[int, str], b: Union[int, str]) -> None:
     """
     f does the thing. The arguments can either be ints or strings but they must
     both have the same type.
@@ -666,7 +675,7 @@ class mod.TestClassAttributeDocs
 class TestClassAttributeDocs:
     """A class"""
 
-    code: Optional[CodeType]  # noqa: UP007
+    code: Optional[CodeType]
     """An attribute"""
 
 
@@ -717,7 +726,7 @@ mod.func_with_parameters_and_stuff_after(a, b)
    More info about the function here.
 """,
 )
-def func_with_parameters_and_stuff_after(a: int, b: int) -> int:  # noqa: ARG001
+def func_with_parameters_and_stuff_after(a: int, b: int) -> int:
     """A func
 
     :param a: a tells us something
@@ -751,7 +760,7 @@ mod.func_with_rtype_in_weird_spot(a, b)
       int
 """,
 )
-def func_with_rtype_in_weird_spot(a: int, b: int) -> int:  # noqa: ARG001
+def func_with_rtype_in_weird_spot(a: int, b: int) -> int:
     """A func
 
     :param a: a tells us something
@@ -799,7 +808,7 @@ mod.empty_line_between_parameters(a, b)
    More stuff here.
 """,
 )
-def empty_line_between_parameters(a: int, b: int) -> int:  # noqa: ARG001
+def empty_line_between_parameters(a: int, b: int) -> int:
     """A func
 
     :param a: One of the following possibilities:
@@ -992,7 +1001,7 @@ def napoleon_returns() -> CodeType:
 
     """,
 )
-def google_docstrings(arg1: CodeType, arg2: ModuleType) -> CodeType:  # noqa: ARG001
+def google_docstrings(arg1: CodeType, arg2: ModuleType) -> CodeType:
     """Summary line.
 
     Extended description of function.
@@ -1024,7 +1033,7 @@ def google_docstrings(arg1: CodeType, arg2: ModuleType) -> CodeType:  # noqa: AR
 
     """,
 )
-def docstring_with_multiline_note_after_params(param: int) -> None:  # noqa: ARG001
+def docstring_with_multiline_note_after_params(param: int) -> None:
     """Do something.
 
     Args:
@@ -1055,7 +1064,7 @@ def docstring_with_multiline_note_after_params(param: int) -> None:  # noqa: ARG
 
     """,
 )
-def docstring_with_bullet_list_after_params(param: int) -> None:  # noqa: ARG001
+def docstring_with_bullet_list_after_params(param: int) -> None:
     """Do something.
 
     Args:
@@ -1084,11 +1093,11 @@ def docstring_with_bullet_list_after_params(param: int) -> None:  # noqa: ARG001
           maybe multiple lines
 
        Next Term
-          Somethign about it
+          Something about it
 
     """,
 )
-def docstring_with_definition_list_after_params(param: int) -> None:  # noqa: ARG001
+def docstring_with_definition_list_after_params(param: int) -> None:
     """Do something.
 
     Args:
@@ -1100,7 +1109,7 @@ def docstring_with_definition_list_after_params(param: int) -> None:  # noqa: AR
         maybe multiple lines
 
     Next Term
-        Somethign about it
+        Something about it
     """
 
 
@@ -1122,7 +1131,7 @@ def docstring_with_definition_list_after_params(param: int) -> None:  # noqa: AR
 
     """,
 )
-def docstring_with_enum_list_after_params(param: int) -> None:  # noqa: ARG001
+def docstring_with_enum_list_after_params(param: int) -> None:
     """Do something.
 
     Args:
@@ -1152,12 +1161,12 @@ def docstring_with_enum_list_after_params(param: int) -> None:  # noqa: ARG001
           maybe multiple lines
 
        Next Term
-          Somethign about it
+          Something about it
 
        -[ Example ]-
     """,
 )
-def docstring_with_definition_list_after_params_no_blank_line(param: int) -> None:  # noqa: ARG001
+def docstring_with_definition_list_after_params_no_blank_line(param: int) -> None:
     """Do something.
 
     Args:
@@ -1169,11 +1178,57 @@ def docstring_with_definition_list_after_params_no_blank_line(param: int) -> Non
         maybe multiple lines
 
     Next Term
-        Somethign about it
+        Something about it
     .. rubric:: Example
     """
 
 
+@expected(
+    """
+    mod.has_typevar(param)
+
+       Do something.
+
+       Parameters:
+          **param** ("TypeVar"("T")) -- A parameter.
+
+       Return type:
+          "TypeVar"("T")
+
+    """,
+)
+def has_typevar(param: T) -> T:
+    """Do something.
+
+    Args:
+        param: A parameter.
+    """
+    return param
+
+
+@expected(
+    """
+    mod.has_newtype(param)
+
+       Do something.
+
+       Parameters:
+          **param** ("NewType"("W", "str")) -- A parameter.
+
+       Return type:
+          "NewType"("W", "str")
+
+    """,
+)
+def has_newtype(param: W) -> W:
+    """Do something.
+
+    Args:
+        param: A parameter.
+    """
+    return param
+
+
 AUTO_FUNCTION = ".. autofunction:: mod.{}"
 AUTO_CLASS = """\
 .. autoclass:: mod.{}
@@ -1185,14 +1240,205 @@ AUTO_EXCEPTION = """\
 """
 
 
+@expected(
+    """
+    mod.typehints_use_signature(a: AsyncGenerator) -> AsyncGenerator
+
+       Do something.
+
+       Parameters:
+          **a** ("AsyncGenerator") -- blah
+
+       Return type:
+          "AsyncGenerator"
+
+    """,
+    typehints_use_signature=True,
+    typehints_use_signature_return=True,
+)
+def typehints_use_signature(a: AsyncGenerator) -> AsyncGenerator:
+    """Do something.
+
+    Args:
+        a: blah
+    """
+    return a
+
+
+prolog = """
+.. |test_node_start| replace:: {test_node_start}
+""".format(test_node_start="test_start")
+
+
+@expected(
+    """
+    mod.docstring_with_multiline_note_after_params_prolog_replace(param)
+
+       Do something.
+
+       Parameters:
+          **param** ("int") -- A parameter.
+
+       Return type:
+          "None"
+
+       Note:
+
+         Some notes. test_start More notes
+
+    """,
+    rst_prolog=prolog,
+)
+def docstring_with_multiline_note_after_params_prolog_replace(param: int) -> None:
+    """Do something.
+
+    Args:
+        param: A parameter.
+
+    Note:
+
+        Some notes. |test_node_start|
+        More notes
+    """
+
+
+epilog = """
+.. |test_node_end| replace:: {test_node_end}
+""".format(test_node_end="test_end")
+
+
+@expected(
+    """
+    mod.docstring_with_multiline_note_after_params_epilog_replace(param)
+
+       Do something.
+
+       Parameters:
+          **param** ("int") -- A parameter.
+
+       Return type:
+          "None"
+
+       Note:
+
+         Some notes. test_end More notes
+
+    """,
+    rst_epilog=epilog,
+)
+def docstring_with_multiline_note_after_params_epilog_replace(param: int) -> None:
+    """Do something.
+
+    Args:
+        param: A parameter.
+
+    Note:
+
+        Some notes. |test_node_end|
+        More notes
+    """
+
+
+@expected(
+    """
+    mod.docstring_with_see_also()
+
+       Return type:
+          "str"
+
+       See also: more info at <https://example.com>`_.
+
+    """
+)
+def docstring_with_see_also() -> str:
+    """
+    .. seealso:: more info at <https://example.com>`_.
+    """
+    return ""
+
+
+@expected(
+    """
+    mod.has_doctest1()
+
+       Test that we place the return type correctly when the function has
+       a doctest.
+
+       Return type:
+          "None"
+
+       >>> this is a fake doctest
+       a
+       >>> more doctest
+       b
+    """
+)
+def has_doctest1() -> None:
+    r"""Test that we place the return type correctly when the function has a doctest.
+
+    >>> this is a fake doctest
+    a
+    >>> more doctest
+    b
+    """
+
+
+Unformatted = TypeVar("Unformatted")
+
+
+@warns("cannot cache unpickable configuration value: 'typehints_formatter'")
+@expected(
+    """
+    mod.typehints_formatter_applied_to_signature(param: Formatted) -> Formatted
+
+       Do nothing
+
+       Parameters:
+          **param** (Formatted) -- A parameter
+
+       Return type:
+          Formatted
+
+       Returns:
+          The return value
+    """,
+    typehints_use_signature=True,
+    typehints_use_signature_return=True,
+    typehints_formatter=lambda _, __=None: "Formatted",
+)
+def typehints_formatter_applied_to_signature(param: Unformatted) -> Unformatted:
+    """
+    Do nothing
+
+    Args:
+        param: A parameter
+
+    Returns:
+        The return value
+    """
+    return param
+
+
+# Config settings for each test run.
+# Config Name: Sphinx Options as Dict.
+configs = {
+    "default_conf": {},
+    "prolog_conf": {"rst_prolog": prolog},
+    "epilog_conf": {
+        "rst_epilog": epilog,
+    },
+    "bothlog_conf": {
+        "rst_prolog": prolog,
+        "rst_epilog": epilog,
+    },
+}
+
+
 @pytest.mark.parametrize("val", [x for x in globals().values() if hasattr(x, "EXPECTED")])
+@pytest.mark.parametrize("conf_run", ["default_conf", "prolog_conf", "epilog_conf", "bothlog_conf"])
 @pytest.mark.sphinx("text", testroot="integration")
 def test_integration(
-    app: SphinxTestApp,
-    status: StringIO,
-    warning: StringIO,
-    monkeypatch: pytest.MonkeyPatch,
-    val: Any,
+    app: SphinxTestApp, status: StringIO, warning: StringIO, monkeypatch: pytest.MonkeyPatch, val: Any, conf_run: str
 ) -> None:
     if isclass(val) and issubclass(val, BaseException):
         template = AUTO_EXCEPTION
@@ -1202,6 +1448,8 @@ def test_integration(
         template = AUTO_FUNCTION
 
     (Path(app.srcdir) / "index.rst").write_text(template.format(val.__name__))
+    app.config.__dict__.update(configs[conf_run])
+    app.config.__dict__.update(val.OPTIONS)
     monkeypatch.setitem(sys.modules, "mod", sys.modules[__name__])
     app.build()
     assert "build succeeded" in status.getvalue()  # Build succeeded
